<?php
namespace FM\Controller;

use FM\Controller\FMCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use FM\Service\VistoriasService;
use FM\Traits\TResponse;

class VistoriasController extends FMCrud{

    use TResponse;
    
    protected $dataSourceName = 'fcm_checklistagend';
    protected $VistoriasService;

    public function __construct($crudService, $environment, VistoriasService $VistoriasService) {
        parent::__construct($crudService, $environment);
        $this->VistoriasService = $VistoriasService;
        $this->Environment = $environment;
    }

    // Busca nome e local da vistoria
    public function getInspectionNameAndLocal(DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();

        $var = $this->VistoriasService->getInspectionNameAndLocal($params['id']);
        $dtsVal = new DataSet('info', $var);
        $response->addDataSet($dtsVal);
    }

    // Busca as vistorias e algumas informações entras
    public function getInspectionsInfo(DTO\Request $request, DTO\Response $response) {
        try {
            $var = $this->VistoriasService->getInspectionsInfo();
            $dtsVal = new DataSet('info', $var);
            $response->addDataSet($dtsVal);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getVistorias(DTO\Request $request, DTO\Response $response) {
        try {
            $data = $this->VistoriasService->getVistorias();
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getCriteria(DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();

        $var = $this->VistoriasService->getCriteria($params['id']);
        $dtsVal = new DataSet('info', $var);
        $response->addDataSet($dtsVal);
    }

    // Busca Vistorias para edição
    public function getInspectionToUpdate(DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();

        $nrgerenciamentochk = $params['params']['NRGERENCIAMENTOCHK'];
        $nratividadechk = $params['params']['NRATIVIDADECHK'];

        $var = $this->VistoriasService->getInspectionToUpdate($nrgerenciamentochk, $nratividadechk);
        $dtsVal = new DataSet('info', $var);
        $response->addDataSet($dtsVal);
    }

    // Busca as atividades da Vistoria
     public function getActivities(DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();

        $id = $params['id'];

        $var = $this->VistoriasService->getActivities($id);
        $dtsVal = new DataSet('info', $var);
        $response->addDataSet($dtsVal);
    }

    // Salva as informações de Vistoria
    public function updateInspection(DTO\Request $request, DTO\Response $response) {
        try {
            // $request->getParameters() quando for passar parametro colocar na função abaixo
              $data = $this->VistoriasService->updateInspection($request->getParameters());
              $this->sendResponse($data, 200);
          } catch (Exception $ex) {
              $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
          }
    }

    // Responsável por baixar o aquivo do fileserver
    public function downloadDocFileServer(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrseqmoddocfct = $row["NRSEQMODDOCFCT"];
            $nrorg = $this->Environment->getNrOrg();

            $result = $this->VistoriasService->downloadDocFileServer($nrseqmoddocfct, $nrorg);
            $response->addDataSet(new DataSet("URLDOCUMENTO", array($result)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getConformities(DTO\Request $request, DTO\Response $response) {
        $parametro = $request->getParameters();
        $variavel = $this->VistoriasService->getConformities($parametro);
        $dtsVal = new DataSet('criterias', [$variavel]);
        $response->addDataSet($dtsVal);
    }
}
?>