<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class FuncionariosHierarqFactory {
    
    use TQuery;

    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    // Busca toda a ramificação de usuários da hierarquias
    public function getHierarchyDetails($nrhierarquiafunc) {
        $sql = "SELECT A.NRFUNCIONARIOSHIERARQ, A.CDOPERADOR, A.CDOPERADORPAI, B.NMOPERADOR, H.NMHIERARQUIAFUNC
                        FROM FCM_FUNCIONARIOSHIERARQ A
                        JOIN OPERADOR B ON B.CDOPERADOR = A.CDOPERADOR
                        JOIN FCM_HIERARQUIAFUNC H ON H.NRHIERARQUIAFUNC = A.NRHIERARQUIAFUNC
                       WHERE A.NRHIERARQUIAFUNC = {$nrhierarquiafunc}";

        $std = $this->connection->prepare($sql);
        $std->execute();
        return $std->fetchAll();
    }

    public function getHierarquiaFacilities($parametros) {

        $sql = "SELECT A.CDOPERADOR
                        FROM FCM_FUNCIONARIOSHIERARQ A
                        JOIN OPERADOR B ON B.CDOPERADOR = A.CDOPERADOR
                        JOIN FCM_HIERARQUIAFUNC H ON H.NRHIERARQUIAFUNC = A.NRHIERARQUIAFUNC
                       WHERE A.NRHIERARQUIAFUNC = (SELECT NRHIERARQUIAFUNC
                                                    FROM FCM_HIERARQUIAFUNC
                                                    WHERE IDATIVO = 'S')
                         AND A.CDOPERADOR IN (SELECT CDOPERADOR
                                                        FROM FCM_FUNCIONARIOSHIERARQ
                                                        START WITH CDOPERADOR     = :CDOPERADOR
                                                        CONNECT BY PRIOR CDOPERADOR = CDOPERADORPAI)";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
        return $std->fetchAll();
    }
    
    // Busca uma lista personalizada de usuários que podem ser adicionados na hierarquia
    public function getHierarchyUsers($nrhierarquiafunc) {
        $sql = "SELECT CDOPERADOR, NMOPERADOR
                        FROM OPERADOR
                    WHERE CDOPERADOR NOT IN( SELECT A.CDOPERADOR
                                                                FROM OPERADOR A
                                                                    JOIN FCM_FUNCIONARIOSHIERARQ B ON  B.CDOPERADOR = A.CDOPERADOR
                                                    WHERE B.NRHIERARQUIAFUNC = {$nrhierarquiafunc})";

        $std = $this->connection->prepare($sql);

        $std->execute();
        return $std->fetchAll();
    }
   
   // Busca uma lista personalizada de usuários que podem ser Pais na hierarquia 
    public function getHierarchyUsersParents($nrhierarquiafunc) {
        $sql = "SELECT CDOPERADOR, NMOPERADOR
                        FROM OPERADOR
                    WHERE CDOPERADOR IN(SELECT A.CDOPERADOR
                                                    FROM OPERADOR A
                                                    JOIN FCM_FUNCIONARIOSHIERARQ B ON  B.CDOPERADOR = A.CDOPERADOR
                                                    WHERE B.NRHIERARQUIAFUNC = {$nrhierarquiafunc})";

        $std = $this->connection->prepare($sql);

        $std->execute();
        return $std->fetchAll();
    }
    
    // Insere um novo usuário à hierarquia
    public function insertHierarchyUser($params) {
        $sql = "INSERT INTO FCM_FUNCIONARIOSHIERARQ (NRFUNCIONARIOSHIERARQ, NRHIERARQUIAFUNC, CDOPERADOR,
                                                    CDOPERADORPAI, DTINCLUSAO)
                    VALUES (:NRFUNCIONARIOSHIERARQ, :NRHIERARQUIAFUNC, :CDOPERADOR,
                            :CDOPERADORPAI, SYSDATE)";

        $std = $this->connection->prepare($sql);

        $std->execute($params);
    }
    
    // Deleta um usuário e sua ramificação abaixo
    public function deleteHierarchyUser($cdoperador) {

        $sql = "DELETE FROM FCM_FUNCIONARIOSHIERARQ
                    WHERE CDOPERADOR = {$cdoperador}";

        $std = $this->connection->prepare($sql);
        $std->execute();
    }
}
?>