<?php
namespace FM\Service;

use FM\Factory\VistoriasFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\Remote\FileUploader\Server;
use Zeedhi\Framework\Remote\cURLRequest;
use FM\Service\FuncionariosHierarqService;

class VistoriasService {

    protected $connection;
    protected $VistoriasFactory;
    protected $FuncionariosHierarqService;
    protected $Environment;
    protected $newCode;

    public function __construct (EntityManager $entityManager, VistoriasFactory $VistoriasFactory, FuncionariosHierarqService $FuncionariosHierarqService, Environment $Environment, NewCode $newCode) {
        $this->connection                   = $entityManager->getConnection();
        $this->VistoriasFactory             = $VistoriasFactory;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
    }
    
    public function getConformities($params) {

        $parametros = array(
            "NRGERENCIAMENTOCHK"    => $params['NRGERENCIAMENTOCHK'],
            "NRATIVIDADECHK"        => $params['NRATIVIDADECHK'],
        );

        $tasks = $this->VistoriasFactory->getInspectionToUpdate($parametros);
        
        $paramReturn = array(
            "IDSITUACAOGERCHK" => $tasks[0]['IDSITUACAOGERCHK'],
            "IDSITUACAOAGEND"  => $tasks[0]['IDSITUACAOAGEND'],
            "DSOBSERVACAO"     => [],
            "CRITERIOS"        => [],
            "CRITERIOSATIVOS"  => [],
        );
        
        foreach ($tasks as $task) {
            $criterias = $this->VistoriasFactory->getConformities(array(
                "NRTAREFAGERCHK" => $task["NRTAREFAGERCHK"]
                ));
                if (!in_array($task["DSOBSTAREFAGERCHK"], $paramReturn["DSOBSERVACAO"])) {
                    $paramReturn["DSOBSERVACAO"][] =  $task["DSOBSTAREFAGERCHK"];
                }
            foreach($criterias as $criteria) {
                if (!in_array($criteria["NMCRITERIOTARGERCHK"], $paramReturn["CRITERIOS"])) {
                    $paramReturn["CRITERIOS"][] =  $criteria["NMCRITERIOTARGERCHK"];
                }
                if ($criteria["IDMARCADO"] == "S" && !in_array($criteria["NMCRITERIOTARGERCHK"], $paramReturn["CRITERIOSATIVOS"])) {
                     $paramReturn["CRITERIOSATIVOS"][] =  $criteria["NMCRITERIOTARGERCHK"];
                }
            }
        }

       $paramReturn["DSOBSERVACAO"] = implode(',', $paramReturn["DSOBSERVACAO"]);
       return $paramReturn;
    }
    
    public function getInspectionsInfo() {

        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);

        return $this->VistoriasFactory->getInspectionsInfo($listaOperadores);
    }

    public function getVistorias() {

        $cdoperador = $this->Environment->getCdOperador();
        $listaOperadores = $this->FuncionariosHierarqService->montarHierarquiaOperadores($cdoperador);

        return $this->VistoriasFactory->getVistorias($listaOperadores);
    }

    // Busca os critérios
    public function getCriteria($params) {
        $operators = $this->VistoriasFactory->getCriteria($params);
        return $operators;
    }

    public function getInspectionNameAndLocal($params) {
        $operators = $this->VistoriasFactory->getInspectionNameAndLocal($params);
        return $operators;
    }

    // Busca as informações da vistoria
    public function getInspectionToUpdate($nrgerenciamentochk, $nratividadechk) {
        $param = array (
            'NRGERENCIAMENTOCHK' => $nrgerenciamentochk,
            'NRATIVIDADECHK' => $nratividadechk
        );

        $operators = $this->VistoriasFactory->getInspectionToUpdate($param);
        return $operators;
    }

    // Edita a vistoria
    public function updateInspection($param) {
        $this->connection->beginTransaction();
        try {
            
            forEach($param["dataBase"] as $paramatro) {
                
                // limpa toda ves que chama a função caso tenha alterado algo 
                $limpaItens = array(
                    "IDMARCADO"          => "N",
                    "NRGERENCIAMENTOCHK" => $paramatro['NRGERENCIAMENTOCHK'],
                    "NRATIVIDADECHK"     => $paramatro['NRATIVIDADECHK']
                );
                
                $this->VistoriasFactory->updateCriterio($limpaItens, '');
                
                $updateNotas = array(
                    'NRGERENCIAMENTOCHK' => $paramatro['NRGERENCIAMENTOCHK'],
                    'NRATIVIDADECHK' => $paramatro['NRATIVIDADECHK'],
                    'DSOBSTAREFAGERCHK'  => ''
                );
 
                $this->VistoriasFactory->updateNote($updateNotas);
                
                // atualiza novamente todos os itens;
                if ($paramatro["valueSwitch"] != true ){
                    
                    $arrayNomeCriterio = explode(',', $paramatro['valuesChkbox']);
                    
                    forEach ( $arrayNomeCriterio as $criterios){
                        
                        $updateVistoria = array(
                            "IDMARCADO"          => "S",
                            "NRGERENCIAMENTOCHK" => $paramatro['NRGERENCIAMENTOCHK'],
                            "NRATIVIDADECHK"     => $paramatro['NRATIVIDADECHK'],
                            "NMCRITERIOTARGERCHK" => $criterios,
                        );
                        
                        $paramQuery = " AND NMCRITERIOTARGERCHK = :NMCRITERIOTARGERCHK";
                        
                        $this->VistoriasFactory->updateCriterio($updateVistoria, $paramQuery);
                        
                        $updateNotas = array(
                            'NRGERENCIAMENTOCHK' => $paramatro['NRGERENCIAMENTOCHK'],
                            'NRATIVIDADECHK' => $paramatro['NRATIVIDADECHK'],
                            'DSOBSTAREFAGERCHK'  => $paramatro['valueObservation']
                        );
 
                        $this->VistoriasFactory->updateNote($updateNotas);
                        
                    }
                    
                    $parametros = array(
                        "IDSITUACAOGERCHK"   => 3,
                        "NRGERENCIAMENTOCHK" => $paramatro['NRGERENCIAMENTOCHK'],
                    );
    
                    $this->VistoriasFactory->upStatusInspection($parametros);
                    
                } else {
                    
                    $updateNotas = array(
                        'NRGERENCIAMENTOCHK' => $paramatro['NRGERENCIAMENTOCHK'],
                        'NRATIVIDADECHK' => $paramatro['NRATIVIDADECHK'],
                        'DSOBSTAREFAGERCHK'  => $paramatro['valueObservation']
                    );
 
                   $this->VistoriasFactory->updateNote($updateNotas);
                }
            }
            $updateSituationSurvey =  array(
                'NRGERENCIAMENTOCHK' => $paramatro['NRGERENCIAMENTOCHK'],
                'IDSITUACAOGERCHK' => '3',
            );

            $this->VistoriasFactory->upStatusInspection($updateSituationSurvey);
            

        $this->connection->commit();

        } catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    // Busca as atividades da vistoria
    public function getActivities($id) {

        $operators = $this->VistoriasFactory->getActivities($id);
        return $operators;
    }


    // Baixa do fileServe o arquivo de documento e salva em uma pasta temporária
    public function downloadDocFileServer($nrseqmoddocfct, $nrorg) {
        $result = array();
        $params = array(
            "NRSEQMODDOCFCT" => $nrseqmoddocfct,
            "NRORG"          => $nrorg
        );

        $documento = $this->modeloDocFactory->getDocumento($params);
        if(count($documento) > 0) {
            $microtime = microtime(true);
            $microtime = str_replace(".", "_", $microtime);

            $documento = $documento[0];
            $fileDir   = $this->fileServerBaseUrl . $documento["DSCAMINHOARQ"];
            $fileName  = $microtime . "_" . str_replace(",", "_", $documento["NMARQUIVO"]);

            $fileDirTemp = $this->FileServerService->downloadDocFileServer($fileDir, $fileName, $nrorg);
            $fileName    = base64_encode($fileName);
            $result = "/downloadArquivo?requestType=Row&row[FILENAME]=" . $fileName . "&row[FILETYPE]=" . $documento["DSTIPO"];
        }

        return $result;
    }

}
?>