/*
 Tecnisa - Tecnologia em Informtica

 Nome: SUGESTAO_APURA_FATU

  Descrio:
  Funo para retornar as sugestes de perodo para a apurao do faturamento gesto direta

 Criao:
 08/02/2013 - Marcelo Alexsandro Souza Silva
*/

DROP TYPE SUGESTAO_FAT_TABLE
/

DROP TYPE SUGESTAO_FAT_TYPE
/

CREATE TYPE SUGESTAO_FAT_TYPE AS
OBJECT (NRORG       NUMBER,
        CDCLIENTE   VARCHAR2(21),
        CDFILIAL    VARCHAR2(4),
        DTINIAPURAC DATE,
        DTFIMAPURAC DATE,
        DTSAIDAPUR  DATE,
        NRAPURACFAT VARCHAR2(10),
        IDBLOQDATA  VARCHAR2(1)
)
/

CREATE TYPE SUGESTAO_FAT_TABLE AS
TABLE OF SUGESTAO_FAT_TYPE
/

CREATE OR REPLACE FUNCTION SUGESTAO_APURA_FATU
(P_NRORG     IN NUMBER,
 P_CDFILIAL  IN FILIAL.CDFILIAL%TYPE,
 P_CDCLIENTE IN CLIENTE.CDCLIENTE%TYPE) RETURN SUGESTAO_FAT_TABLE PIPELINED
IS
   V_EXISTEAPURACAO FLOAT;
   V_DTPROCESS      DATE;
   V_IDUTILDTPROC   CHAR(1);
   V_COUNT          FLOAT;
   OUT_RECORD       SUGESTAO_FAT_TYPE := SUGESTAO_FAT_TYPE (NULL, NULL, NULL, NULL,
                                                            NULL, NULL, NULL, NULL);
BEGIN
   V_EXISTEAPURACAO := 0;
   FOR C_PRINCIPAL IN
     (SELECT
         C.CDCLIENTE,
         TRUNC(DTINIAPURAC) DTINIAPURAC,
         TRUNC(DTFIMAPURAC) DTFIMAPURAC,
         IDBLOQUEADO,
         C.CDFILIAL,
         C.NRAPURACFAT,
         N.DTENTRSAID AS DTSAIDAPUR
       FROM CTRLAPURFAT C, NOTAFISCAL N
       WHERE C.NRORG          = P_NRORG
         AND C.CDFILIAPUR     = P_CDFILIAL
         AND C.CDCLIENTE      = P_CDCLIENTE
         AND N.NRORG(+)       = C.NRORG
         AND N.CDFILIAL(+)    = C.CDFILIAL
         AND N.NRAPURACFAT(+) = C.NRAPURACFAT
         AND C.DTINIAPURAC = (SELECT MAX(DTINIAPURAC)
                              FROM CTRLAPURFAT
                              WHERE NRORG        = P_NRORG       AND
                                    CDFILIAPUR   = P_CDFILIAL  AND
                                    CDCLIENTE    = P_CDCLIENTE AND
                                    NVL(CDTIPOFATUCT,'001') = '001')
         AND C.NRAPURACFAT = (SELECT MAX(NRAPURACFAT)
                              FROM CTRLAPURFAT
                              WHERE NRORG      = P_NRORG
                              AND CDFILIAPUR   = P_CDFILIAL
                              AND CDCLIENTE    = P_CDCLIENTE))
   LOOP
      V_EXISTEAPURACAO := 1;
      IF C_PRINCIPAL.IDBLOQUEADO = 'S' THEN
         OUT_RECORD.NRORG       := P_NRORG;
         OUT_RECORD.CDCLIENTE   := P_CDCLIENTE;
         OUT_RECORD.CDFILIAL    := NULL;
         OUT_RECORD.DTINIAPURAC := C_PRINCIPAL.DTFIMAPURAC + 1;
         OUT_RECORD.DTFIMAPURAC := NULL;
         OUT_RECORD.DTSAIDAPUR  := NULL;
         OUT_RECORD.NRAPURACFAT := NULL;
         OUT_RECORD.IDBLOQDATA  := 'S';
      ELSE
         V_COUNT := 0;
         SELECT COUNT(1)
           INTO V_COUNT
           FROM CTRLAPURFAT
          WHERE NRORG = P_NRORG
            AND CDFILIAL = P_CDFILIAL
            AND CDCLIENTE = C_PRINCIPAL.CDCLIENTE
            AND NVL(CDTIPOFATUCT, '001') = '001';
         IF V_COUNT < 2 THEN
            OUT_RECORD.NRORG       := P_NRORG;
            OUT_RECORD.CDCLIENTE   := P_CDCLIENTE;
            OUT_RECORD.CDFILIAL    := C_PRINCIPAL.CDFILIAL;
            OUT_RECORD.DTINIAPURAC := C_PRINCIPAL.DTINIAPURAC;
            OUT_RECORD.DTFIMAPURAC := C_PRINCIPAL.DTFIMAPURAC;
            OUT_RECORD.DTSAIDAPUR  := C_PRINCIPAL.DTSAIDAPUR;
            OUT_RECORD.NRAPURACFAT := C_PRINCIPAL.NRAPURACFAT;
            OUT_RECORD.IDBLOQDATA  := 'N';
         ELSE
            OUT_RECORD.NRORG       := P_NRORG;
            OUT_RECORD.CDCLIENTE   := P_CDCLIENTE;
            OUT_RECORD.CDFILIAL    := C_PRINCIPAL.CDFILIAL;
            OUT_RECORD.DTINIAPURAC := C_PRINCIPAL.DTINIAPURAC;
            OUT_RECORD.DTFIMAPURAC := C_PRINCIPAL.DTFIMAPURAC;
            OUT_RECORD.DTSAIDAPUR  := C_PRINCIPAL.DTSAIDAPUR;
            OUT_RECORD.NRAPURACFAT := C_PRINCIPAL.NRAPURACFAT;
            OUT_RECORD.IDBLOQDATA  := 'S';
         END IF;
      END IF;
   END LOOP;
   IF V_EXISTEAPURACAO = 0 THEN
      SELECT NVL(MAX(IDUTILDTPROC), 'N')
      INTO V_IDUTILDTPROC
      FROM PARAMGERAL;

      SELECT DECODE(V_IDUTILDTPROC,  'S', NVL(MAX(PF.DTPROCESSA), TRUNC(SYSDATE)), TRUNC(SYSDATE))
        INTO V_DTPROCESS
        FROM PARAMFILIAL PF
       WHERE PF.CDFILIAL = P_CDFILIAL;
       OUT_RECORD.NRORG       := P_NRORG;
       OUT_RECORD.CDCLIENTE   := P_CDCLIENTE;
       OUT_RECORD.CDFILIAL    := NULL;
       OUT_RECORD.DTINIAPURAC := V_DTPROCESS;
       OUT_RECORD.DTFIMAPURAC := NULL;
       OUT_RECORD.DTSAIDAPUR  := NULL;
       OUT_RECORD.NRAPURACFAT := NULL;
       OUT_RECORD.IDBLOQDATA  := 'N';
   END IF;
   PIPE ROW (OUT_RECORD);
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('SUGESTAO_APURA_FATU','5.50.000'); END;
/
