/* eslint-disable */
import {
  IDictionary, Http, Metadata, IEventParam, Loader, Router, SimpleDatasource,
} from '@zeedhi/core';
import {
  Select, Container, Text, Card, Textarea, DialogService, Col, Date, CheckboxMultiple, IComponentRender, Button, AlertService,
  Form, TextInput, FileInput, Switch, Modal, ModalService, LoadingService,
} from '@zeedhi/common';
import {
  IterableCarousel, CardTitle, TekGrid,
} from '@zeedhi/teknisa-components-common';
import FileSaver from 'file-saver';
import FileUpload from '../FileUpload';
import { getRouteParams } from '../../router';
import { Utilities } from '@/util/Utilities';

export class Fm00005000VistoriasController {
  private anexoModal!: Modal;

  private infoModal!: Modal;

  private fileInputModal!: Modal;

  private ImagesModal!: Modal;

  public hierarquiasController = Loader.getInstance('HierarquiasController');

  public modalController = Loader.getInstance('NovoCriterioController');

  public visibilidade: boolean = true;

  public currentRow : any = {};

  get visibilidadeDoFrame() {
    return !this.visibilidade;
  }

  public visualizarDadosGerais({ row }: any) {
    this.visibilidade = false;
    this.currentRow = row;

    Metadata.getInstance<Text>('inspectionDtNrgerenciamentochk').text = row.NRGERENCIAMENTOCHK;

    this.getInspectionNameAndLocal(row.NRGERENCIAMENTOCHK);
    this.getActivities(row.NRGERENCIAMENTOCHK);
  }

  public voltarPaginaInicial() {
    this.visibilidade = true;
  }


  /* ---------------------- Tela Vistorias  --------------------- */

  // Variável que abriga o cards atividades
  public selected: IDictionary = [];

  public nomeCardClicado: string = '';

  public inspectionName: string = '';

  public quantidadeAtividades: Number = 0;

  public dataBase: IDictionary = [];

  public nrAtividadeChk: Number = -1;

  // Recebe cada info do banco e retorna para o array cima
  public setData(nm: string, id: number) {
    return {
      name: nm,
      value: id,
    };
  }

  // Monta e exibe os cards com as informações de cada vistoria
  public async mountInpectionsCards() {
    const containerObject = Metadata.getInstance<Container>('inpectionsListContainer');
    containerObject.children = [];

    LoadingService.show();
    Http.post('/getInspectionsInfo').then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        containerObject.children.push(< unknown > this.inspectionCard(database.NRGERENCIAMENTOCHK, database.NMGERENCIAMENTOCHK, database.NRCHECKLISTAGEND, database.NMLOCALCLIENTE, database.NMLOCALPAI, database.DTHORAAGEND, database.NMFUNCIONARIO, database.IDSITUACAOGERCHK, database.IDSITUACAOAGEND) as IComponentRender);
      });
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }

  // Manuseia a data vinda do banco
  public timeManager(time: any) {
    const date = `${time[0].split('-')[2]}-${time[0].split('-')[1]}-${time[0].split('-')[0]} - ${time[1].substr(0, 5)}`;
    return date;
  }

  // Seta as cores de Situação
  public statusColor({ component }: IEventParam<Text>) {
    if (component.text === 'Aguardando Vistoria') {
      component.text = "<span  style='color: #ff6700; font-size: 14px;'>AGUARDANDO VISTORIA</span>";
    } else if (component.text === 'Em andamento') {
      component.text = "<span  style='color: #023e8a; font-size: 14px;'>EM ANDAMENTO</span>";
    } else if (component.text === 'Avaliado') {
      component.text = "<span  style='color: #2b9348; font-size: 14px;'>A V A L I A D O</span>";
    } else if (component.text === 'Tarefa Pendente') {
      component.text = "<span style='color: #ff6700; font-size: 14px;'>TAREFA PENDENTE</span>";
    } else {
      component.text = "<span style='color: #FA980D; font-size: 14px;'>CANCELADO</span>";
    }
  }

  // Adiciona as informações dos selects em um array
  public filter() {
    const dateObject = Metadata.getInstance<Date>('inspectionDate').value;
    return {
      'LC.NRTIPOLOCALCLI': Metadata.getInstance<Select>('inspectionLocalType').value,
      'LC.NRLOCALCLIENTE': Metadata.getInstance<Select>('inspectionLocals').value,
      'GCHK.NRCHECKLIST': Metadata.getInstance<Select>('inspectionChecklists').value,
      'GCHK.NRFUNCIONARIO': Metadata.getInstance<Select>('inspectionEmployees').value,
      'GCHK.IDSITUACAOGERCHK': Metadata.getInstance<Select>('inspectionStatus').value,
      'TO_CHAR(GCHK.DTHORAAGEND,\'DD/MM/YYYY\')': ((dateObject != null) ? `TO_CHAR('${dateObject}')` : null),
    };
  }

  // Card de Vistorias
  public inspectionCard(nrgerenciamentochk: number, nmgerenciamentochk: string, nrchecklistagend: number, nmlocalcliente: string, nmlocalclientepai: string, dthoraagend: string, nmfuncionario: string, idsituacaogerchk: number, idsituacaoagend: number) {
    return {
      name: 'ColunaCard',
      component: 'ZdCol',
      cssClass: 'zd-pa-0 spacing-between-cards',
      children: [
        {
          name: `inspection${nrgerenciamentochk}`,
          component: 'ZdCard',
          to: `/vistorias/detalhes/${nrgerenciamentochk}`,
          events: {
            click: '{{Fm00005000VistoriasController.setInspectionToUpdate}}',
          },
          children: [
            {
              name: nmlocalcliente,
              component: 'ZdText',
              text: '',
              isVisible: false,
            },
            {
              name: nrchecklistagend,
              component: 'ZdText',
              text: '',
              isVisible: false,
            },
            {
              name: 'linhaInterna',
              component: 'ZdRow',
              justify: 'space-between',
              align: 'center',
              children: [
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '2',
                  children: [
                    {
                      name: 'linha',
                      component: 'ZdText',
                      text: nmgerenciamentochk,
                      tag: 'h4',
                      cssClass: 'color-title-card',
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: 3,
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      align: 'center',
                      justify: 'center',
                      children: [
                        {
                          name: 'mapButton',
                          component: 'ZdButton',
                          icon: true,
                          iconName: 'mdi-map-marker-outline',
                          style: {
                            cursor: 'default',
                          },
                        },
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: `${nmlocalcliente} | ${nmlocalclientepai}`,
                          style: {
                            color: '#6c757d',
                          },
                        },
                      ],
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '2',
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      align: 'center',
                      justify: 'center',
                      children: [
                        {
                          name: 'mapButton',
                          component: 'ZdButton',
                          icon: true,
                          iconName: 'mdi-clock-check-outline',
                          style: {
                            cursor: 'default',
                          },
                        },
                        {
                          name: 'linha',
                          component: 'ZdText',
                          cssClass: 'zd-pl-1',
                          text: dthoraagend,
                          style: {
                            color: '#6c757d',
                          },
                        },
                      ],
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: '3',
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      align: 'center',
                      justify: 'center',
                      children: [
                        {
                          name: 'mapButton',
                          component: 'ZdButton',
                          icon: true,
                          iconName: 'mdi-account-outline',
                          style: {
                            cursor: 'default',
                          },
                        },
                        {
                          name: 'linha',
                          component: 'ZdText',
                          cssClass: 'zd-pl-1',
                          text: nmfuncionario,
                          style: {
                            color: '#6c757d',
                          },
                        },
                      ],
                    },
                  ],
                },
                {
                  name: 'Col',
                  component: 'ZdCol',
                  cols: 2,
                  children: [
                    {
                      name: 'linhaInterna',
                      component: 'ZdRow',
                      align: 'center',
                      justify: 'center',
                      children: [
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: idsituacaogerchk,
                          style: {
                            'font-size': '15px',
                          },
                          events: {
                            onCreated: '{{Fm00005000VistoriasController.statusColor}}',
                          },
                        },
                        {
                          name: 'linha',
                          component: 'ZdText',
                          text: idsituacaoagend,
                          isVisible: false,
                          style: {
                            'font-size': '15px',
                          },
                        },
                      ],
                    },
                  ],
                },
              ],
            },
          ],
        },
      ],
    };
  }

  /* ---------------------- Tela vistoriasDetalhes  --------------------- */

  public isVisibleElement: boolean = !(localStorage.getItem('idpermisssvistoria') === 'V');

  public isDisabledElement: boolean = (localStorage.getItem('idpermisssvistoria') === 'V');

  // Fecha a caixa de diálogo
  private dialogHideEvents: any = {
    click: () => {
      DialogService.hide();
    },
  };

  // Chama o evento de clear data e fecha a caixa de diálogo
  private dialogConfirmEvents: any = {
    click: () => {
      this.clearInspectionData();
      DialogService.hide();
    },
  };


  private finishConfirmEvents: any = {
    click: () => {
      this.finishInspectionDetails();
      DialogService.hide();
    },
  };

  // Recebe o nome e id da atividade
  public setActivityData(ActName: string, ActId: number) {
    return {
      id: ActId,
      name: ActName,
    };
  }

  // Recebe o nome, id e idmarcado do critério
  public setCriterioData(nm: string, id: number, markedId: string) {
    if (markedId === 'S') Metadata?.getInstance<CheckboxMultiple>('criteriaInspectionDt').value.push(id);
    return {
      name: nm,
      value: id,
      criterioId: markedId,
    };
  }

  public changeCardColor({ component }: IEventParam<Card>) {
    if (component.cssClass === '') {
      component.cssClass = 'inspectionCardClicked';
      component.children[0].color = '#ffffff';

      // const id = component.name.substr(8);
      // const subtitle = component.children[1].text;

      // this.getCriteria(id, subtitle);
    } else { // Se tirar o clicar pela segunda vez tirar a seleção
      component.cssClass = '';
      component.children[0].color = '#0077b6';

      const checkObject = Metadata?.getInstance<CheckboxMultiple>('criteriaInspectionDt');
      checkObject!.datasource!.data = [];
    }
  }

  // Busca nome da Vistoria e Onde é feita
  public getInspectionNameAndLocal(nrgerenciamentochk: string) {
    Http.get(`/getInspectionInfo/${nrgerenciamentochk}/nameAndLocal`).then((response: IDictionary) => {
      response.data.dataset.info.forEach((database: IDictionary) => {
        Metadata.getInstance<Text>('inspectionDtName').text = database.NMGERENCIAMENTOCHK;
        Metadata.getInstance<Text>('inspectionDtLocal').text = database.NMLOCALCLIENTE;
      });
    });
  }

  // Busca critérios
  public getCriteria(id:string, subtitle: string) {
    const params = id;
    const checkObject = Metadata?.getInstance<CheckboxMultiple>('criteriaInspectionDt');
    const cardObject = Metadata.getInstance<CardTitle>('inspectionDtTitleCriteria');
    const btnObject = Metadata.getInstance<Button>('newCriterioinspectionDt');
    cardObject.subtitle = subtitle;
    checkObject!.datasource!.data = [];
    checkObject!.value = [];
    Http.get(`/getCriteria/${params}`).then((response: IDictionary) => {
      btnObject.disabled = (response.data.dataset.info[0].IDSITUACAOAGEND !== '4');
      response.data.dataset.info.forEach((database: IDictionary) => {
        checkObject?.datasource?.data?.push(this.setCriterioData(database.NMCRITERIOTARGERCHK, database.NRCRITERIOTARGERCHK, database.IDMARCADO));
      });
    });
  }

  // Busca as atividades
  public getActivities(id: string) {
    const cardObject = Metadata.getInstance<IterableCarousel>('inspectionDtActivities');
    Http.get(`/getActivities/${id}/Activities`).then((response: IDictionary) => {
      this.quantidadeAtividades = response.data.dataset.info.length;
      response.data.dataset.info.forEach((database: IDictionary) => {
        cardObject.iterable.datasource.data.push(this.setActivityData(database.NMATIVIDADE, database.NRATIVIDADECHK));
      });
    });
  }

  // Método para enviar os dados para update da vistoria.
  public saveSurvey() {
    const dataBase = Object.keys(this.dataBase).map((key: string) => this.dataBase[key]);
    const grid = Metadata.getInstance<TekGrid>('tek-grid-vistorias');

    LoadingService.show();
    Http.post('/updateInspection', { dataBase }).then(() => {
      Utilities.showSuccessMessage();
      this.voltarPaginaInicial();
      grid.datasource.get();
    }).catch((error: any) => {
      Utilities.showErrorMessage(error);
    }).finally(() => {
      LoadingService.hide();
    });
  }

  // Monta os critérios de não conformidades na tela antes de finalizar a tarefa
  public getNoteAndCriteria({ component }: IEventParam<Card>) {
    if (Object.keys(this.selected).length) {
      this.dataBase[this.selected.name] = {
        NRGERENCIAMENTOCHK: Metadata.getInstance<Text>('inspectionDtNrgerenciamentochk').text,
        NRATIVIDADECHK: this.selected.name.substr(8),
        valuesChkbox: Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt').value.map((dados: any) => `${dados}`).join(','),
        valueObservation: Metadata.getInstance<Textarea>('inspectionDtNote').value,
        valueSwitch: Metadata.getInstance<Switch>('basicUsageSwitch').value,
      };
      this.selected.cssClass = 'inspectionCard';
    }
    component.cssClass = 'inpectionCardClicked';
    this.selected = component;
    const textAreaObject = Metadata.getInstance<Textarea>('inspectionDtNote');
    const criteriaTitleObject = Metadata.getInstance<Text>('inspectionDtCriteriaTitle');
    const cardObject = Metadata.getInstance<CardTitle>('inspectionDtTitleCriteria');
    const btnObject = Metadata.getInstance<Button>('newCriterioinspectionDt');
    cardObject.subtitle = component.children[1].text;
    textAreaObject.disabled = this.isDisabledElement;
    const chkbox = Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt');
    chkbox.readonly = false;
    criteriaTitleObject.text = '- Marque apenas campos de não conformidade:';
    this.nrAtividadeChk = parseInt(component.name.substr(8), 10);
    Http.post('/getConformities', {
      NRATIVIDADECHK: component.name.substr(8),
      NRGERENCIAMENTOCHK: Metadata.getInstance<Text>('inspectionDtNrgerenciamentochk').text,
    }).then((response: IDictionary) => {
      response.data.dataset.criterias.forEach((criterias: IDictionary) => {
        if (criterias.IDSITUACAOGERCHK === '3' || criterias.IDSITUACAOAGEND !== '4') {
          this.desactivateButtons();
        } else if (criterias.IDSITUACAOGERCHK !== '3' && criterias.IDSITUACAOAGEND !== '4') {
          this.desactivateButtons();
        }
        btnObject.disabled = false;

        if (chkbox.datasource instanceof SimpleDatasource) {
          chkbox!.datasource!.items = criterias.CRITERIOS;
        }

        if (Object.keys(this.dataBase).includes(component.name)) {
          chkbox.value = this.dataBase[component.name].valuesChkbox.replaceAll('\'', '').split(',');
          Metadata.getInstance<Textarea>('inspectionDtNote').value = this.dataBase[component.name].valueObservation;
          Metadata.getInstance<Switch>('basicUsageSwitch').value = this.dataBase[component.name].valueSwitch;
          this.hasCriterion();
        } else {
          chkbox.value = criterias.CRITERIOSATIVOS;
          Metadata.getInstance<Textarea>('inspectionDtNote').value = criterias.DSOBSERVACAO;
          Metadata.getInstance<Switch>('basicUsageSwitch').value = false;
          this.hasCriterion();
        }
      });
    });
  }

  // Método para verificar o switch e ativar e desativar os botões
  public hasCriterion() {
    const swtch = Metadata.getInstance<Switch>('basicUsageSwitch');
    if (swtch.value === true) {
      Metadata.getInstance<Textarea>('inspectionDtNote').value = 'Vistoria Realizada, sem não conformidades.';
      Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt').readonly = true;
      Metadata.getInstance<Button>('cancelInspectionDt').disabled = false;
      Metadata.getInstance<Button>('saveInspectionDt').disabled = false;
      Metadata.getInstance<Button>('newCriterioinspectionDt').disabled = true;
      Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt').value = [];
    } else if (swtch.value === false) {
      Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt').readonly = false;
    }
  }

  // Mostra na tela as informações da vistoria
  public setInspectionToUpdate() {
    const routeParams = getRouteParams();
    const { id } = routeParams;

    Metadata.getInstance<Text>('inspectionDtNrgerenciamentochk').text = id;

    this.getInspectionNameAndLocal(id);
    this.getActivities(id);
  }

  public noPermissonModify() {
    const chkbox = Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt');
    const tArea = Metadata.getInstance<Textarea>('inspectionDtNote');
    const saveButton = Metadata.getInstance<Button>('saveInspectionDt');
    const newCriteria = Metadata.getInstance<Button>('newCriterioinspectionDt');
    const fileInput = Metadata.getInstance<FileInput>('inspectionDtImage');

    chkbox.readonly = true;
    tArea.readonly = true;
    saveButton.disabled = true;
    fileInput.disabled = true;
    newCriteria.disabled = true;
  }

  // Salva a avaliação do usuário e só libera a opção de Finalizar após vistoriar todos os locais.
  public saveCriterias() {
    this.dataBase[this.selected.name] = {
      NRGERENCIAMENTOCHK: Metadata.getInstance<Text>('inspectionDtNrgerenciamentochk').text,
      NRATIVIDADECHK: this.selected.name.substr(8),
      valuesChkbox: Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt').value.map((dados: any) => `${dados}`).join(','),
      valueObservation: Metadata.getInstance<Textarea>('inspectionDtNote').value,
      valueSwitch: Metadata.getInstance<Switch>('basicUsageSwitch').value,
    };
    if (Object.keys(this.dataBase).length === this.quantidadeAtividades) {
      AlertService.show({
        name: 'buttons-alert',
        text: 'Atividade avaliada com sucesso!',
        color: 'success',
        top: true,
        left: false,
        timeout: 1500,
      });
      Metadata.getInstance<Button>('FinalizeInspectionButton').disabled = false;
    }
  }

  // Salva as informações de critérios e observação para a atividade selecionada
  // public saveInspectionDetails({ component }: any) {
  //   let idSituation: any;
  //   let message: any;

  //   const params = {
  //     NRGERENCIAMENTOCHK: Metadata.getInstance<Text>('inspectionDtNrgerenciamentochk').text,
  //     DSOBSTAREFAGERCHK: 'Vistoria realizada sem conformidades',
  //     IDSITUACAOGERCHK: '3',
  //     // NRFOTOTAREFAGERCHK: Metadata.getInstance<FileInput>('inspectionDtImage').value,
  //   };

  //   if (component.name === 'saveInspectionDt') {
  //     idSituation = 2;
  //     message = 'Alterações salvas com sucesso!';
  //     Metadata.getInstance<Button>('FinalizeInspectionButton').disabled = false;
  //   } else {
  //     idSituation = 3;
  //     message = 'Vistoria realizada com sucesso!';
  //   }

  //   Http.post('/updateInspection', [idSituation, params]).then((response: IDictionary) => {
  //     const returnMessage = response.data.dataset.info[0];
  //     if (returnMessage !== 'OK') {
  //       DialogService.show({
  //         name: 'warning-dialog',
  //         type: 'warning',
  //         title: 'Erro!',
  //         text: 'Operação bloqueada! Ocorreu um erro ao salvar os dados desta vistoria.<br><br>Se o problema persistir entre em contato com o suporte.',
  //       });
  //     } else if (returnMessage === 'OK') {
  //       Metadata.getInstance<Button>('saveInspectionDt').disabled = true;
  //       Metadata.getInstance<Button>('cancelInspectionDt').disabled = true;

  //       AlertService.show({
  //         name: 'buttons-alert',
  //         text: message,
  //         color: 'success',
  //         top: true,
  //         left: false,
  //         timeout: 1500,
  //       });
  //     }
  //   });
  // }

  // Ativa os botoões de salvar e cancelar
  public activateButtons() {
    const saveButtonObject = Metadata.getInstance<Button>('saveInspectionDt');
    const cancelButtonObject = Metadata.getInstance<Button>('cancelInspectionDt');

    saveButtonObject.disabled = false;
    cancelButtonObject.disabled = false;
  }

  // Desativa os botoões de salvar e cancelar
  public desactivateButtons() {
    const saveButtonObject = Metadata.getInstance<Button>('saveInspectionDt');
    const cancelButtonObject = Metadata.getInstance<Button>('cancelInspectionDt');

    saveButtonObject.disabled = true;
    cancelButtonObject.disabled = true;
  }

  // Limpa o dados da tela
  public clearInspectionData() {
    const chkObject = Metadata?.getInstance<CheckboxMultiple>('criteriaInspectionDt');
    const textAreaObject = Metadata.getInstance<Textarea>('inspectionDtNote');
    const fileInputObject = Metadata.getInstance<FileInput>('inspectionDtImage');
    const cardTitleObject = Metadata.getInstance<CardTitle>('inspectionDtTitleCriteria');
    const criteriaTitleObject = Metadata.getInstance<Text>('inspectionDtCriteriaTitle');
    const newCriterioObject = Metadata.getInstance<Button>('newCriterioinspectionDt');

    chkObject!.datasource!.data = [];
    textAreaObject.value = null;
    textAreaObject.disabled = true;
    fileInputObject.value = true;
    fileInputObject.disabled = true;
    cardTitleObject.subtitle = '';
    criteriaTitleObject.text = '- Nenhuma atividade selecionada.';
    newCriterioObject.disabled = true;

    const saveButtonObject = Metadata.getInstance<Button>('saveInspectionDt');
    const cancelButtonObject = Metadata.getInstance<Button>('cancelInspectionDt');

    saveButtonObject.disabled = true;
    cancelButtonObject.disabled = true;

    this.clearActivitiesColor();
  }

  // Tira a seleção dos cards de atividade
  public clearActivitiesColor() {
    const iterableCardObject = Metadata.getInstance<IterableCarousel>('inspectionDtActivities');
    iterableCardObject.children.forEach((activity: IDictionary) => {
      activity.color = '#ffffff';
    });
  }

  // Volta para tela de vistorias
  public finishInspectionDetails() {
    Router.replace('/vistorias');
  }

  // Confirma se quer cancelar
  public confirmCancelButton() {
    DialogService.show({
      name: 'cancelButton',
      type: 'warning',
      title: 'Cancelar',
      text: 'Deseja cancelar suas alterações?',
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: this.dialogHideEvents,
          outline: true,
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: this.dialogConfirmEvents,
        },
      ],
    });
  }

  // Confirma se quer salvar
  public confirmSaveButton() {
    DialogService.show({
      name: 'cancelButton',
      type: 'warning',
      title: 'Salvar',
      text: 'Deseja salvar suas alterações para esta atividade?',
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: this.dialogHideEvents,
          outline: true,
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
        },
      ],
    });
  }

  // Confirma se quer deixar a página
  public confirmFinishButton() {
    DialogService.show({
      name: 'finishButton',
      type: 'warning',
      title: 'Concluir',
      text: 'Todas as alterações não salvas serão perdidas. Você deseja mesmo continuar?',
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: this.finishConfirmEvents,
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: this.dialogHideEvents,
        },
      ],
    });
  }

  // Tratamento do modal para o botão "novo critério"
  public createModal() {
    this.modalController.createBasicUsageModal();
  }

  public destroyModal() {
    this.modalController.destroyBasicUsageModal();
  }

  public openModal() {
    Promise.all([this.modalController.openBasicModal()]).then(() => {
      const buttonObject = Metadata.getInstance<Button>('closeModalButton');
      buttonObject.events = {
        click: () => this.newCriterio(),
      };
    });
  }

  public fecharModal() {
    this.modalController.closeBasicModal();
  }

  // Adiciona um novo critério
  public newCriterio() {
    const form = Metadata.getInstance<Form>('modalCriacaoCriterio');
    if (form.validate()) {
      const chkObject = Metadata.getInstance<CheckboxMultiple>('criteriaInspectionDt');
      const criterioName = Metadata.getInstance<TextInput>('InputNovoCriterio').value;
      chkObject!.datasource!.data!.push({
        name: criterioName,
        value: null,
      });
      this.fecharModal();
    }
  }

  // Teste Adicionar foto
  public uploadPictures() {
    const fileInputObject = Metadata.getInstance<FileInput>('inspectionDtImage').value;
    const types: IDictionary = ['image/jpg', 'image/jpeg', 'image/png'];
    const buildFilesPromises = fileInputObject.map((file: any) => {
      if (types.includes(file.type)) {
        return FileUpload.buildFileSendObject(file);
      }
      return null;
    });
    Promise.all(buildFilesPromises)
      .then((filesPromise: any) => {
        const routeParams = getRouteParams();
        const idVistoria = routeParams.id;
        const body = { NRGERENCIAMENTOCHK: idVistoria, NRATIVIDADECHK: this.nrAtividadeChk, files: filesPromise };
        Http.post('/uploadPictures', body)
          .then(() => {
            AlertService.show({
              name: 'basic-alert',
              text: 'Arquivos enviados com sucesso',
              color: 'success',
              top: true,
              left: false,
              timeout: 3000,
            });
          })
          .catch((err) => {
            AlertService.show({
              name: 'buttons-alert',
              text: err,
              color: 'error',
              top: true,
              left: false,
              timeout: 1500,
            });
          });
      })
      .catch((err) => {
        AlertService.show({
          name: 'buttons-alert',
          text: err,
          color: 'error',
          top: true,
          left: false,
          timeout: 1500,
        });
      });
  }

  // https://github.com/eligrey/FileSaver.js
  public downloadFile() {
    const url2 = 'http://midiatest.teknisa.com/files/tkOrg205cc0d590e9d43/foto_vistoria_fm/610180333fdfc.png';
    Http.post('/DownloadFile', {
      DSFOTOTAREFAGERCHK: url2,
    })
      .then((response) => {
        const value = response.data.dataset.content[0];
        const arrayBuffer = atob(value);
        const blob = new Blob([arrayBuffer], { type: 'image/png' });
        FileSaver.saveAs(blob, '610180333fdfc.png');
      })
      .catch((err) => {
        AlertService.show({
          name: 'buttons-alert',
          text: err,
          color: 'error',
          top: true,
          left: false,
          timeout: 1500,
        });
      });
  }

  public consoleLog(variavel: any) {
    const titulo = Metadata.getInstance<Text>('titulo');
    titulo.text = variavel;
  }

  public async creatanexoModal() {
    this.anexoModal = await ModalService.create({
      name: 'anexoModal',
      grid: {
        sm: 3,
      },
      children: [
        {
          name: 'fromTypescriptHeader',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: 'Oque você deseja fazer?',
              cssClass: 'corPadrao',
              cssStyle: {
                'font-size': '1.4em',
                'font-weight': 'bold',
              },
            },
          ],
          rightSlot: [
            {
              name: 'closeModalButton',
              component: 'ZdButton',
              iconName: 'mdi-close',
              icon: true,
              events: {
                click: '{{Fm00005000VistoriasController.closanexoModal}}',
              },
            },
          ],
        },
        {
          name: 'form-anexo',
          component: 'ZdForm',
          cssClass: 'formAnexo',
          children: [
            {
              name: 'Btn-anexar-AnexoModal',
              label: 'Anexar Imagens',
              component: 'ZdButton',
              cssClass: 'btnCamera',
              events: {
                click: '{{Fm00005000VistoriasController.creatfileInputModal}}',
              },
            },
            {
              name: 'Btn-visualizarAnexo-AnexoModal',
              label: 'Exibir Imagens anexadas',
              outline: true,
              component: 'ZdButton',
              cssClass: 'btnCamera',
              events: {
                click: '{{Fm00005000VistoriasController.creatImagesModal}}',
              },
            },
          ],
        },
      ],
    });
    this.anexoModal.show();
  }

  public async closanexoModal() {
    await this.anexoModal.hide();
    this.anexoModal?.destroy();
  }

  public async creatImagesModal() {
    /* eslint-disable */
    this.ImagesModal = await ModalService.create({
      name: 'ImagesModal',
      children: [
        {
          name: 'HeaderImagesModal',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: 'Imagens anexada',
              cssClass: 'corPadrao',
              cssStyle: {
                'font-size': '1.4em',
                'font-weight': 'bold',
              },
            },
          ],
          rightSlot: [
            {
              name: 'closeModalButton',
              component: 'ZdButton',
              iconName: 'mdi-close',
              icon: true,
              events: {
                click: '{{Fm00005000VistoriasController.closImagesModal}}',
              },
            },
          ],
        },
        {
          name: 'form-anexo',
          component: 'ZdForm',
          cssClass: 'formAnexo',
          children: [
            {
              name: 'carouselImagesAnexoTask',
              component: 'TekIterableCarousel',
              center: true,
              slidesPerView: 1,
              infiniteScroll: false,
              events: {
                onCreated: '{{Fm00005000VistoriasController.getImage}}',
                onMounted: ({ element }) => {
                  if (element) {
                    setTimeout(() => { 
                      const hooper: any = element.querySelector('.hooper');
                      const hooperObj = hooper.__vue__;
                      hooperObj.updateWidth();
                    }, 300);              
                  }
                },
              },
              iterable: {
                componentMetadata: {
                  name: 'LinhaContendoImagensAtivos_<<row.id>>',
                  component: 'ZdRow',
                  justify: 'center',
                  children: [
                    {
                      name: `imagemDoAtivo_'<<row.NRFOTOTAREFAGERCHK>>'`,
                      component: 'ZdImage',
                      src: '<<row.DSFOTOTAREFAGERCHK>>',
                      width: '35vw',
                      cssClass: 'maxHeightImage',
                    },
                  ],
                },
              },
            },
          ],
        },
        {
          name: 'RowWithButtonDeleteImage',
          component: 'ZdRow',
          justify: 'center',
          children: [
            {
              name: 'Btn-DeleteImage',
              component: 'ZdButton',
              label: 'Apagar imagem',
              width: '95%',
              events:{
                click: '{{Fm00005000VistoriasController.deletImage}}',
              }
            }
          ]
        },
        { 
          name: 'SemResultadoImagesVistoria',
          component: 'ZdText',
          text: 'Nenhuma foto anexada.',
          cssClass: 'conteudo-cards-menores noData zd-mb-6',
          isVisible: false,
        }
      ],
    });
    this.ImagesModal.show();
  }

  public async closImagesModal() {
    await this.ImagesModal.hide();
    this.ImagesModal?.destroy();
  }

  public getImage({ component }: any) {
    LoadingService.show('Abrindo Imagens...');
    Http.get('/getFotoVistoria?NRTAREFA='+window.location.pathname.split('/')[3]).then((response: any) => {
      LoadingService.hide();
      component.iterable.datasource.data = response.data;
      if(response.data.length == 0) {
        Metadata.getInstance<Button>('Btn-DeleteImage').isVisible = false;
        Metadata.getInstance<Text>('SemResultadoImagesVistoria').isVisible = true;
        this.ImagesModal.grid.lg = 3;
      }
    })
  }

  public async creatfileInputModal() {
    this.fileInputModal = await ModalService.create({
      name: 'fileInputModal',
      grid: {
        sm: 5,
      },
      children: [
        {
          name: 'HeaderFileInputModal',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: 'Anexar imagens',
              cssClass: 'corPadrao',
              cssStyle: {
                'font-size': '1.4em',
                'font-weight': 'bold',
              },
            },
          ],
          rightSlot: [
            {
              name: 'closeModalButton',
              component: 'ZdButton',
              iconName: 'mdi-close',
              icon: true,
              events: {
                click: '{{Fm00005000VistoriasController.closfileInputModal}}',
              },
            },
          ],
        },
        {
          name: 'form-anexo',
          component: 'ZdForm',
          cssClass: 'formAnexo',
          children: [
            {
              name: 'FileInput-ImagensDoAtivo_carouselImagesAnexoTask',
              component: 'ZdFileInput',
              showDragArea: true,
              dragAreaHeight: '300px',
              multiple: true,
              showBorder: false,
              chips: true,
              removableChip: false,
              accept: '.png,.jpg,.jpeg',
              events: {
                change: '{{Fm00005000VistoriasController.exibirImagem}}',
              },
            },
            {
              name: 'LinhaCarrosel',
              component: 'ZdCol',
              children: [
                {
                  name: 'carouselImagesAnexoTask',
                  component: 'TekIterableCarousel',
                  slidesPerView: 1,
                  infiniteScroll: false,
                  iterable: {
                    datasource: {
                      uniqueKey: 'id',
                      data: [],
                    },
                    componentMetadata: {
                      name: 'LinhaContendoImagensAtivos_<<row.id>>',
                      component: 'ZdRow',
                      justify: 'center',
                      children: [
                        {
                          name: 'imagemDoAtivo_<<row.id>>',
                          component: 'ZdImage',
                          src: '<<row.src>>',
                          width: '10vw',
                        },
                      ],
                    },
                  },
                },
              ],
              events: {
                onMounted: '{{Fm00005000VistoriasController.invisivel}}',
              },
            },
            {
              name: 'Btn-fileInputModal',
              label: 'Anexar',
              component: 'ZdButton',
              width: '95%',
              events: {
                click: '{{Fm00005000VistoriasController.anexarImage}}',
              },
            },
          ],
        },
      ],
    });
    this.fileInputModal.show();
  }

  public async closfileInputModal() {
    await this.fileInputModal.hide();
    this.fileInputModal?.destroy();
  }

  public async anexarImage() {
    const imagensConvertidas = await FileUpload.converterImagens('FileInput-ImagensDoAtivo_carouselImagesAnexoTask');
    if (imagensConvertidas.length > 0) {
      LoadingService.show('Salvando...');
      Http.post('/anexarFotoVistoria', {
          IMAGEM: imagensConvertidas,
          NRTAREFA: window.location.pathname.split('/')[3],
        }).then((response: any) => {
        LoadingService.hide();
        if('success' === response.data){
          const file = Metadata.getInstance<FileInput>('FileInput-ImagensDoAtivo_carouselImagesAnexoTask');
          file.value = [];
          this.exibirImagem({ component: file });
          this.messages('OK', 'Imagens anexadas com sucesso!')
        } else {
          this.messages('Error', 'Ocorreu um erro ao salvar as imagens.<br>.Tente novamente.<br><br>Se o problema persistir entre em contato com o suporte.')
        }
      });
    }
  }

  public invisivel({ component }: any) {
    component.isVisible = false;
  }

  public exibirImagem({ component }: any) {
    const carrosel = Metadata.getInstance<IterableCarousel>(component.name.split('_')[1]);
    const row = Metadata.getInstance<Col>('LinhaCarrosel');

    const imagens: any = [];
    if (component.value.length) {
      row.isVisible = true;
      component.value.forEach((imagem: any, pos: number) => {
        imagens.push({
          id: `cliente_${pos}`,
          src: window.URL.createObjectURL(imagem),
        });
      });
    } else {
      row.isVisible = false;
    }
    carrosel.iterable.datasource.updateData(imagens);
  }

  public messages(returnMessage: string, mensagem: string) {
    if (returnMessage === 'OK') {
      AlertService.show({
        name: 'basic-alert',
        text: mensagem,
        top: true,
        left: false,
        timeout: 3000,
        color: 'success',
      });
    } else {
      DialogService.show({
        name: 'warning-dialog',
        type: 'warning',
        title: 'Erro!',
        text: mensagem,
      });
    }
  }

  public async deletImage(){
    LoadingService.show('Deletando...');
    const carrosel = Metadata.getInstance<IterableCarousel>('carouselImagesAnexoTask');
    let img = carrosel.iterable.datasource.data;
    const [deleteData] = img.filter((value: any, index: any) => { return index === carrosel.currentSlide });
    Http.get(`/deleteFotoVistoria?NRTAREFA=${window.location.pathname.split('/')[3]}&&NRFOTOTAREFA=${deleteData.NRFOTOTAREFAAGEND || deleteData.NRFOTOTAREFAGERCHK}`).then((response: any) => {
      LoadingService.hide();
      if('success' == response.data) {
        const newData = img.filter((value: any, index: any) => { return index !== carrosel.currentSlide });
        carrosel.iterable.datasource.updateData(newData);
        if(newData.length == 0) {
          Metadata.getInstance<Button>('Btn-DeleteImage').isVisible = false;
          Metadata.getInstance<Text>('SemResultadoImagesVistoria').isVisible = true;
          this.ImagesModal.grid.lg = 3;
        }
      }
    });
  }

  public async creatInfoModal() {
    LoadingService.show('Carregando informações...');
    let dadosBanco: any;
    await Http.get(`/getitensDetailsTaskForVistoria?NR=${window.location.pathname.split('/')[3]}`).then(async (response: any) => {
      dadosBanco = response.data.dataset.details;
      LoadingService.hide();
      this.infoModal = await ModalService.create({
        name: 'infoModal',
        grid: {
          sm: 5,
        },
        children: [
          {
            name: 'HeaderinfoModal',
            component: 'ZdHeader',
            color: 'transparent',
            cssClass: 'headerSemPadding',
            padless: true,
            elevation: 0,
            leftSlot: [
              {
                name: 'titleText',
                component: 'ZdText',
                text: 'Dados da Tarefa',
                cssClass: 'corPadrao',
                cssStyle: {
                  'font-size': '1.4em',
                  'font-weight': 'bold',
                },
              },
            ],
            rightSlot: [
              {
                name: 'closeModalButton',
                component: 'ZdButton',
                iconName: 'mdi-close',
                icon: true,
                events: {
                  click: '{{Fm00005000VistoriasController.closinfoModal}}',
                },
              },
            ],
          },
          {
            name: 'form-anexo',
            component: 'ZdForm',
            style: {
              'max-height': '78vh',
              'overflow': 'hidden scroll',
            },
            children: [
              {
                name: 'TitleCriterio',
                component: 'ZdText',
                text: 'Critérios',
                cssClass: 'zd-mb-0',
                style: {
                  color: '#495057',
                  'font-size': '1.2em',
                }
              },
              {
                name: 'RowWithCriterios',
                component: 'ZdRow',
                style: {
                  'min-height': '75px',
                  'max-height': '140px',
                  'overflow-y': 'scroll',
                  'margin': '0 15px 0 0',
                },
                children: [
                  {
                    name: 'basic-usage-example-checkbox-multiple',
                    component: 'ZdCheckboxMultiple',
                    readonly: true,
                    value: dadosBanco.criterion.filter((elemento: any) => elemento.IDREALIZADO === 'S').map((elemento: any) => elemento.NMTAREFA),
                    datasource: {
                      type: 'simple',
                      items: dadosBanco.criterion.map((elemento: any) => elemento.NMTAREFA),
                    }
                  }
                ]
              },
              {
                name: 'FieldObservatio',
                component: 'ZdTextarea',
                label: 'Observações',
                cssClass: 'zd-mr-4 zd-mt-4',
                readonly: true,
                value: dadosBanco.observation.map((elemento: any) => elemento.DSOBSTAREFAAGEND+' ').toString()
              },
              {
                name: 'TitleImages',
                component: 'ZdText',
                text: 'Imagens anexadas',
                style: {
                  color: '#495057',
                  'font-size': '1.2em',
                }
              },
              {
                name: 'carouselImagesAnexoTask',
                component: 'TekIterableCarousel',
                center: true,
                slidesPerView: 1,
                infiniteScroll: false,
                events: {
                  onCreated: ({ component }) => {
                    console.log(component);
                    LoadingService.show('Abrindo Imagens...');
                    Http.get('/getFotoTaskVistoria?NRTAREFA='+window.location.pathname.split('/')[3]).then((response: any) => {
                      LoadingService.hide();
                      component.iterable.datasource.data = response.data;
                      if(response.data.length == 0) {
                        component.isVisible = false;
                        Metadata.getInstance<Text>('SemResultadoImagesTarefasDaVistoria').isVisible = true;
                      }
                    })
                  },
                  onMounted: ({ element }) => {
                    if (element) {
                      setTimeout(() => { 
                        const hooper: any = element.querySelector('.hooper');
                        const hooperObj = hooper.__vue__;
                        hooperObj.updateWidth();
                      }, 300);              
                    }
                  },
                },
                iterable: {
                  componentMetadata: {
                    name: 'LinhaContendoImagensAtivos_<<row.id>>',
                    component: 'ZdRow',
                    justify: 'center',
                    children: [
                      {
                        name: `imagemDoAtivo_'<<row.NRFOTOTAREFAGERCHK>>'`,
                        component: 'ZdImage',
                        src: '<<row.DSFOTOTAREFAGERCHK>>',
                        width: '20vw',
                        cssClass: 'maxHeightImage',
                      },
                    ],
                  },
                },
              },
              { 
                name: 'SemResultadoImagesTarefasDaVistoria',
                component: 'ZdText',
                text: 'Nenhuma foto anexada.',
                cssClass: 'conteudo-cards-menores noData zd-mb-6',
                isVisible: false,
              },
            ],
          },
        ],
      });
      this.infoModal.show();
    });
  }

  public async closinfoModal() {
    await this.infoModal.hide();
    this.infoModal?.destroy();
  }
}
